/*
 * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <doca_log.h>
#include <doca_rmax.h>

#include <doca_utils.h>

#include "rmax_common.h"
#include "common.h"

DOCA_LOG_REGISTER(RMAX_CREATE_STREAM::MAIN);

/* Sample's logic */
doca_error_t rmax_create_stream(struct doca_pci_bdf *pcie_addr, struct program_core_objects *state, struct rmax_stream_config *stream_config);

/*
 * Initialize default configurations; stream and flow configurations that should be used when setting their attributes
 *
 * @config [out]: all needed configuration that should be set.
 */
static void
init_config(struct rmax_stream_config *config)
{
	union {
		const struct in_addr ip;
		uint8_t octets[4];
	} src_addr, dst_addr;

	src_addr.octets[0] = 192;
	src_addr.octets[1] = 168;
	src_addr.octets[2] = 105;
	src_addr.octets[3] = 3;

	dst_addr.octets[0] = 192;
	dst_addr.octets[1] = 168;
	dst_addr.octets[2] = 105;
	dst_addr.octets[3] = 2;

	config->type = DOCA_RMAX_IN_STREAM_TYPE_GENERIC;
	config->scatter_type = DOCA_RMAX_IN_STREAM_SCATTER_TYPE_RAW;
	config->src_ip.s_addr = src_addr.ip.s_addr;
	config->dst_ip.s_addr = dst_addr.ip.s_addr;
	config->dst_port = 5200;
	config->hdr_size = 0;
	config->data_size = 1500;
	config->num_elements = 1024;

	strcpy(config->pci_address, "03:00.0");
}

/*
 * Sample main function
 *
 * @argc [in]: command line arguments size
 * @argv [in]: array of command line arguments
 * @return: EXIT_SUCCESS on success and EXIT_FAILURE otherwise
 */
int
main(int argc, char **argv)
{
	doca_error_t result;
	int exit_status = EXIT_SUCCESS;
	struct doca_pci_bdf pcie_addr;
	struct program_core_objects state;
	struct rmax_stream_config stream_config;
	struct doca_logger_backend *stdout_logger = NULL;

	/* Create a logger backend that prints to the standard output */
	result = doca_log_create_file_backend(stdout, &stdout_logger);
	if (result != DOCA_SUCCESS)
		return EXIT_FAILURE;

	/* default configurations */
	init_config(&stream_config);

	/* ARGP initialization */
	result = doca_argp_init("doca_stream_create", &stream_config);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to init ARGP resources: %s", doca_get_error_string(result));
		return EXIT_FAILURE;
	}

	/* register ARGP parameters of the sample */
	result = register_create_stream_params();
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to register ARGP params: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* start parsing received arguments */
	result = doca_argp_start(argc, argv);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse sample command line argument(s): %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* parse PCIe address */
	result = doca_pci_bdf_from_string(stream_config.pci_address, &pcie_addr);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse PCI address: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* Sample's main logic */
	result = rmax_create_stream(&pcie_addr, &state, &stream_config);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("RMAX create stream sample encountered errors: %s", doca_get_error_string(result));
		exit_status = EXIT_FAILURE;
	}

	return exit_status;
}
