/*
 * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <assert.h>
#include <stdbool.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <netinet/in.h>

#include "rmax_common.h"

DOCA_LOG_REGISTER(RMAX_CREATE_STREAM);

/*
 * Run create_stream sample, which creates stream
 *
 * @pcie_addr [in]: pci address struct for doca device
 * @state [in]: DOCA core related objects
 * @stream_config [in]: stream configurations
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise.
 */
doca_error_t
rmax_create_stream(struct doca_pci_bdf *pcie_addr, struct program_core_objects *state, struct rmax_stream_config *stream_config)
{
	struct doca_rmax_in_stream *stream = NULL;
	struct doca_buf *buf = NULL;
	struct doca_rmax_flow *flow = NULL;
	doca_error_t result;
	struct doca_rmax_job_rx_data job;
	struct doca_event ev;
	uint32_t flags = 0;
	uint16_t stride_size;
	bool run_main_loop;

	/* open DOCA device with the given PCI address */
	result = open_doca_device_with_pci(pcie_addr, NULL, &state->dev);
	if (result != DOCA_SUCCESS)
		return result;

	/* DOCA RMAX library Initialization */
	result = doca_rmax_init();
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to initialize DOCA RMAX library: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* create stream object */
	result = doca_rmax_in_stream_create(&stream);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to create DOCA Rivermax stream: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* set stream attributes, based on received or default configurations */
	result = rmax_stream_set_attributes(stream, stream_config);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to set stream attributes: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* convert DOCA Rmax stream to DOCA context */
	state->ctx = doca_rmax_in_stream_as_ctx(stream);
	if (state->ctx == NULL) {
		DOCA_LOG_ERR("Failed to convert stream to a context");
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* create core DOCA objects */
	result = create_core_objects(state, 1);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to create DOCA core related objects: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* allocate buffers for received data */
	result = rmax_stream_allocate_buf(state, stream, stream_config, &buf, &stride_size);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to allocate RX buffers: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* start DOCA objects */
	result = start_context(state, 1);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to start DOCA objects: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* create a flow using DOCA Rmax API */
	result = doca_rmax_flow_create(&flow);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to create a flow: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* create a DOCA Rmax flow and attach it to the given stream  */
	result = rmax_flow_set_attributes(stream_config, flow);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to set flow attributes: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* Attach flow to a stream using DOCA Rmax API */
	result = doca_rmax_flow_attach(flow, stream);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to attach a flow: %s", doca_get_error_string(result));
		rmax_create_stream_cleanup(state, stream, flow, buf);
		return result;
	}

	/* Prepare initial job */
	job.base.ctx = state->ctx;
	job.base.flags = 0;
	job.base.type = DOCA_RMAX_ACTION_TYPE_RX_DATA;
	job.base.user_data.u64 = 0;

	/* Submit initial job */
	result = doca_workq_submit(state->workq, &job.base);
	if (result == DOCA_SUCCESS)
		run_main_loop = true;
	else {
		DOCA_LOG_ERR("Failed to submit job: %s", doca_get_error_string(result));
		run_main_loop = false;
	}

	while (run_main_loop) {
		result = doca_workq_progress_retrieve(state->workq, &ev, flags);

		switch (result) {
		case DOCA_SUCCESS:
			if ((int)(ev.type) == (int)(DOCA_RMAX_ACTION_TYPE_RX_DATA)) {
				struct doca_rmax_in_stream_completion *comp = (struct doca_rmax_in_stream_completion *)ev.result.ptr;

				if (comp->elements_count > 0) {
					uint8_t *data;

					DOCA_LOG_INFO("Received %4d packet(s), first %lu last %lu",
							comp->elements_count,
							comp->ts_first,
							comp->ts_last);
					/* dump packets */
					data = comp->memblk_ptr_arr[0];
					for (size_t i = 0; i < comp->elements_count; ++i, data += stride_size) {
						char *dump = hex_dump(data, stream_config->data_size);

						DOCA_DLOG_INFO("Packet:\n%s", dump);
						free(dump);
					}
				}
				/* Submit job to receive next chunk of data */
				result = doca_workq_submit(state->workq, &job.base);
				if (result != DOCA_SUCCESS) {
					DOCA_LOG_ERR("Failed to submit job: %s", doca_get_error_string(result));
					break;
				}
			}
			continue;
		case DOCA_ERROR_AGAIN:
			/* no data available */
			continue;
		case DOCA_ERROR_IO_FAILED:
			if ((int)(ev.type) == (int)(DOCA_RMAX_ACTION_TYPE_RX_DATA)) {
				struct doca_rmax_stream_error *err = (struct doca_rmax_stream_error *)ev.result.ptr;

				assert(err);
				DOCA_DLOG_ERR("Error: code=%d message=%s", err->code, err->message);
				break;
			}
			/* fallthrough */
		default:
			DOCA_LOG_ERR("DOCA command failed with code %s", doca_get_error_name(result));
			abort();
		}

		run_main_loop = false;
	}

	/* detach flow */
	result = doca_rmax_flow_detach(flow, stream);
	if (result != DOCA_SUCCESS)
		DOCA_LOG_ERR("Failed to detach flow from stream: %s", doca_get_error_string(result));

	/* Clean and destroy all relevant objects */
	rmax_create_stream_cleanup(state, stream, flow, buf);

	return DOCA_SUCCESS;
}
