/*
 * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <doca_log.h>

#include <doca_utils.h>

#include "rmax_common.h"

DOCA_LOG_REGISTER(RMAX_SET_CLOCK::MAIN);

/* Sample's logic */
doca_error_t set_clock(struct doca_pci_bdf *pcie_addr, struct program_core_objects *state);

/*
 * ARGP Callback - Handle PCI device address parameter
 *
 * @param [in]: Input parameter
 * @opaque [in/out]: PCIe adddress
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
static doca_error_t
pci_address_callback(void *param, void *opaque)
{
	char *pcie_addr = (char *)opaque;
	char *pci_address = (char *)param;
	int len;

	len = strnlen(pci_address, PCI_ADDR_LEN);
	if (len == PCI_ADDR_LEN) {
		DOCA_LOG_ERR("PCI address too long, max %d", USER_PCI_ADDR_LEN);
		return DOCA_ERROR_INVALID_VALUE;
	}
	strcpy(pcie_addr, pci_address);
	return DOCA_SUCCESS;
}

/*
 * Register the command line parameter for the sample.
 *
 * @return: DOCA_SUCCESS on success and DOCA_ERROR otherwise
 */
doca_error_t
register_set_clock_params(void)
{
	doca_error_t result;
	struct doca_argp_param *pci_param;

	result = doca_argp_param_create(&pci_param);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to create ARGP param: %s", doca_get_error_string(result));
		return result;
	}
	doca_argp_param_set_short_name(pci_param, "p");
	doca_argp_param_set_long_name(pci_param, "pci-addr");
	doca_argp_param_set_description(pci_param, "PCI device address");
	doca_argp_param_set_callback(pci_param, pci_address_callback);
	doca_argp_param_set_type(pci_param, DOCA_ARGP_TYPE_STRING);
	result = doca_argp_register_param(pci_param);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to register program param: %s", doca_get_error_string(result));
		return result;
	}

	return DOCA_SUCCESS;
}

/*
 * Sample main function
 *
 * @argc [in]: command line arguments size
 * @argv [in]: array of command line arguments
 * @return: EXIT_SUCCESS on success and EXIT_FAILURE otherwise
 */
int
main(int argc, char **argv)
{
	doca_error_t result;
	int exit_status = EXIT_SUCCESS;
	struct doca_pci_bdf pcie_addr;
	struct program_core_objects state;
	char pcie_address[PCI_ADDR_LEN];
	struct doca_logger_backend *stdout_logger = NULL;

	strcpy(pcie_address, "03:00.0");

	/* Create a logger backend that prints to the standard output */
	result = doca_log_create_file_backend(stdout, &stdout_logger);
	if (result != DOCA_SUCCESS)
		return EXIT_FAILURE;

	/* ARGP initialization */
	result = doca_argp_init("doca_set_clock", &pcie_address);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to init ARGP resources: %s", doca_get_error_string(result));
		return EXIT_FAILURE;
	}

	/* register ARGP parameters of the sample */
	result = register_set_clock_params();
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to register ARGP params: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* start parsing received arguments */
	result = doca_argp_start(argc, argv);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse sample input: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* parse PCIe address */
	result = doca_pci_bdf_from_string(pcie_address, &pcie_addr);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse PCI address: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* Sample's main logic */
	result = set_clock(&pcie_addr, &state);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("RMAX set PTP clock sample encountered errors: %s", doca_get_error_string(result));
		exit_status = EXIT_FAILURE;
	}

	return exit_status;
}
